package com.example.ReactiveWebFlux;


import org.assertj.core.api.Assertions;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.test.context.junit4.SpringRunner;
import org.springframework.test.web.reactive.server.WebTestClient;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;

import java.time.LocalDateTime;

import static org.junit.Assert.assertEquals;
import static org.mockito.ArgumentMatchers.any;
import static org.mockito.ArgumentMatchers.anyString;
import static org.mockito.Mockito.when;
/*
Reference for handler and router Testcases
https://blog.knoldus.com/spring-webflux-testing-your-router-functions-with-webtestclient/
*/

@SpringBootTest(webEnvironment = SpringBootTest.WebEnvironment.RANDOM_PORT)
@RunWith(SpringRunner.class)
public class UserComponentTest {

    @MockBean
    private UserServiceImpl userService;

    @Autowired
    private WebTestClient webTestClient;

    @Before
    public void setUp () {

    }

    @Test
    public void saveUserTest () {
        // ArgumentCaptor<User> userArgumentCaptor = ArgumentCaptor.forClass(User.class);

        User user = new User("5ebbfa934fffbf123cbdf233", "Praneeth", "praneeth@gmail.com", LocalDateTime.now());

        when(userService.saveUser(any())).thenReturn(Mono.just(user));
        webTestClient.post().uri("/user/create")
                .body(Mono.just(getUser()), User.class)
                .exchange()
                .expectStatus().isOk()
                .expectBody(User.class)
                .value(userResponse -> {
                           Assertions.assertThat(userResponse.getId()).isEqualTo("5ebbfa934fffbf123cbdf233");
                           Assertions.assertThat(userResponse.getName()).isEqualTo("Praneeth");
                           Assertions.assertThat(userResponse.getEmail()).isEqualTo("praneeth@gmail.com");
                       }
                );

    }

    private User getUser () {
        User user = new User();
        user.setEmail("praneeth@gmail.com");
        user.setName("praneeth");
        return user;
    }


    @Test
    public void testGetUserById () {

        User user = new User("5ebbac8c470a341gh872cf3", "Adi", "adu@gmail.com", LocalDateTime.now());

        when(userService.getUserById(anyString())).thenReturn(Mono.just(user));
        webTestClient.get().uri("/user/5ebbac8c470a341gh872cf3")
                .exchange()
                .expectStatus().isOk()
                .expectBody(User.class)
                .consumeWith(resp -> {
                    if (resp.getResponseBody() != null) {
                        assertEquals("Adi", resp.getResponseBody().getName());
                    }
                });
    }

    @Test
    public void testGetUsers () {

        User user1 = new User("1", "ABC", "abc@xyz.com", LocalDateTime.now());
        User user2 = new User("2", "XYZ", "xyz@xyz.com", LocalDateTime.now());

        when(userService.getUsers()).thenReturn(Flux.just(user1, user2));

        webTestClient.get()
                .uri("/user")
                .exchange()
                .expectStatus().isOk()
                .expectBodyList(User.class)
                .value(userResponse -> {
                           Assertions.assertThat(userResponse.get(0).getId()).isEqualTo("1");
                           Assertions.assertThat(userResponse.get(0).getName()).isEqualTo("ABC");
                           Assertions.assertThat(userResponse.get(0).getEmail()).isEqualTo("abc@xyz.com");
                           Assertions.assertThat(userResponse.get(1).getId()).isEqualTo("2");
                           Assertions.assertThat(userResponse.get(1).getName()).isEqualTo("XYZ");
                           Assertions.assertThat(userResponse.get(1).getEmail()).isEqualTo("xyz@abc.com");
                       }
                );
    }
}
