package com.example.demo;
import static org.junit.Assert.assertEquals;
import com.example.demo.Repository.EmployeeRepository;
import com.example.demo.model.Employee;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.test.annotation.IfProfileValue;
import org.springframework.test.context.ActiveProfiles;
import org.springframework.test.context.junit4.SpringRunner;
import org.springframework.test.web.reactive.server.WebTestClient;
import org.springframework.web.reactive.function.BodyInserters;

@RunWith(SpringRunner.class)
@SpringBootTest(webEnvironment = SpringBootTest.WebEnvironment.DEFINED_PORT, classes = ReactiveWeb_ReactiveMongo_Webflux.class)
@ActiveProfiles("integration")
public class EmployeeControllerIntegrationTest {

    @Autowired
    WebTestClient webTestClient;

    @Autowired
    EmployeeRepository employeeRepository;
    final private static int port = 8832;
    final private static String baseUrl = "http://localhost:8832";

    @Test
   // @IfProfileValue(name = "test-groups", values = {"integration-test", "critical-test", "smoke-test", "stage-test", "prod-test"})

    public void testSaveEmployee(){
        Employee employee= new Employee("1", "apple", 20000L,null);
        webTestClient.post().uri(baseUrl+"/employee")
                .body(BodyInserters.fromObject(employee))
                .exchange()
                .expectStatus().isOk().expectBody().jsonPath("$.id").isEqualTo("1");

    }
    @Test
    public void testGetEmployeeDetails () {

        webTestClient.get().uri("/employees")
                .exchange()
                .expectStatus().isOk()
                .expectBodyList(Employee.class)
                .consumeWith(resp -> {
                    if (resp.getResponseBody() != null) {
                        assertEquals("Dia", resp.getResponseBody().get(0).getName());
                        assertEquals("Praneeth", resp.getResponseBody().get(1).getName());
                        assertEquals("5ebbfa934fffbf123cbdf233", resp.getResponseBody().get(1).getId());
                    }
                });
    }

    @Test
    public void testGetEmployeeDetailsById () {

        webTestClient.get().uri("/employee/5ebbfa934fffbf123cbdf233")
                .exchange()
                .expectStatus().isOk()
                .expectBody(Employee.class)
                .consumeWith(resp -> {
                    if (resp.getResponseBody() != null) {
                        assertEquals("Praneeth", resp.getResponseBody().getName());
                        assertEquals("5ebbfa934fffbf123cbdf233", resp.getResponseBody().getId());
                    }
                });
    }

    @Test
    public void testGetEmployeeDetailsByIdErrorScenario () {

        webTestClient.get().uri("/employee/5ebbfa934ff123cbdf233")
                .exchange()
                .expectStatus().isOk()
                .expectBody(Employee.class)
                .consumeWith(resp -> {
                    if (resp.getResponseBody() != null) {
                        assertEquals("E100", resp.getResponseBody().getError().getCode());
                        assertEquals("Employee NotFound With given ID", resp.getResponseBody().getError().getMessage());

                    }
                });
    }

    @Test
    public void getAll_UsingWebClientComponentTestCaseMockito() {
        webTestClient.get().uri("/employees").exchange()
                .expectStatus().isOk()
                .expectBodyList(Employee.class)
                .hasSize(3);
        //hasSize(7);
    }


    @Test
    public void testDeleteEmployeeById() {
        webTestClient.delete().uri("/employee/1").exchange()
                .expectStatus().isOk();
        //hasSize(7);
    }
}

