package com.example.demo.Controller;

import com.example.demo.Repository.EmployeeRepository;
import com.example.demo.Repository.EmployeeService;
import com.example.demo.model.Employee;
import com.example.demo.model.Error;
import com.example.demo.model.Profile;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;

import org.springframework.web.reactive.function.client.WebClient;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;
import reactor.util.function.Tuple2;

import java.time.Duration;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;
import java.util.function.Function;
import java.util.stream.Stream;

@RestController
public class EmployeeController {
    private static final Logger LOGGER = LoggerFactory.getLogger(EmployeeController.class);

    WebClient webClient = WebClient.builder()
            .baseUrl("http://localhost:8833/")
            .build();

    @Autowired
    private EmployeeService employeeService;

    @GetMapping(value = "/employees")
    public Mono<List<Employee>> getAll () {

    return employeeService.findAll().switchIfEmpty(generateError1());

    }

    private Mono<List<Employee>> generateError1 () {
        List<Employee> employees = new ArrayList<>();
        employees.add(new Employee(null,null,0L,new Error("E111","No recordes found in db","Np records avaiable")));
        return Mono.just(employees);
    }

    @GetMapping(value = "/employee/{id}")
    public Mono<Employee> getId ( @PathVariable("id") final String empId ) {
        return employeeService.findEmployeeById(empId).switchIfEmpty(generateError());
    }
    private Mono<Employee> generateError () {

        Employee employee = new Employee();
        Error error = new Error("E100", "Employee NotFound With given ID", "record not there in db");
        employee.setError(error);
        return Mono.just(employee);
    }
    @PostMapping("/employee")
    public Mono<Employee> save ( @RequestBody Employee employee ) {

        return employeeService.save(employee).onErrorResume(throwable -> {
            Employee employee1 = new Employee();
            Error error = new Error();
            error.setCode("E100");
            error.setMessage("Error while saving a record");
            error.setDebugMessage(throwable.getMessage());
            employee1.setError(error);
            return Mono.just(employee1);
        });
    }



    @PostMapping(value = "/employees")
    public Flux<Employee> saveAll ( @RequestBody Flux<Employee> entities ) {
        return employeeService.saveAll(entities);
    }

    @DeleteMapping("/employee/{id}")
    public Mono<Void> delete ( @PathVariable("id") final String id ) {
        return employeeService.deleteById(id);
        // employeeRepository
    }

    @GetMapping("/employee/getEmployeeProfileByName/{name}")
    public Mono<Profile> getEmployeeProfileByName(@PathVariable String name){
        Mono<Profile> profile = webClient
                .get()
                .uri("profileByName/"+name)
                .retrieve()
                .bodyToMono(Profile.class);
        profile.subscribe(profile1 -> System.out.println(profile1.toString()));
        //  log.info("Exiting NON-BLOCKING ProfileController!");
        return profile;
    }

    @GetMapping(value = "/employee/getAllProfile",
            produces = MediaType.TEXT_EVENT_STREAM_VALUE)
    public Flux<Profile> getEmployeesNonBlacking() {
        Flux<Profile> profileFlux = webClient
                .get()
                .uri("/profiles")
                .retrieve()
                .bodyToFlux(Profile.class);
                /*.delaySequence(Duration.ofMillis(100))//with backpressure
                .doOnNext(person -> LOGGER.info("Server produces: {}", person));*/

        profileFlux.subscribe(tweet -> System.out.println(tweet.toString()));
        Flux<Long> durationFlux = Flux.interval(Duration.ofSeconds(1));
        return Flux.zip(profileFlux, durationFlux).map(Tuple2::getT1);

    }

    @GetMapping(value = "/getAllProfile",
            produces = MediaType.TEXT_EVENT_STREAM_VALUE)
    public Flux<Profile> getEmployeesBackPressure() {
        Flux<Profile> profileFlux = webClient
                .get()
                .uri("/profiles")
                .retrieve()
                .bodyToFlux(Profile.class)
                .delaySequence(Duration.ofMillis(100))//with backpressure
                .doOnNext(profile -> LOGGER.info("Server produces: {}", profile));

        /*profileFlux.subscribe(tweet -> System.out.println(tweet.toString()));
        Flux<Long> durationFlux = Flux.interval(Duration.ofSeconds(1));
        return Flux.zip(profileFlux, durationFlux).map(Tuple2::getT1);*/
        return profileFlux;

    }

// sample event streaming to get idea
    @GetMapping(produces = MediaType.TEXT_EVENT_STREAM_VALUE,
            value = "/StreamOfEvents")
    Flux<Employee.Event> events () {
        Flux<Employee.Event> eventFlux = Flux.fromStream(Stream.
                generate(() -> new Employee.Event(System.currentTimeMillis(),
                                                  LocalDateTime.now())));
        Flux<Long> durationFlux = Flux.interval(Duration.ofSeconds(5));
        return Flux.zip(eventFlux, durationFlux).map(Tuple2::getT1);
    }
}
