package com.example.demo;

import com.example.demo.Repository.EmployeeRepository;
import com.example.demo.Repository.EmployeeServiceImpl;
import com.example.demo.model.Employee;
import org.junit.FixMethodOrder;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.junit.runners.MethodSorters;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.test.context.junit4.SpringRunner;
import org.springframework.test.web.reactive.server.WebTestClient;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;

import java.util.Collections;

import static org.junit.Assert.assertEquals;
import static org.mockito.ArgumentMatchers.any;
import static org.mockito.ArgumentMatchers.anyString;
import static org.mockito.Mockito.when;

@RunWith(SpringRunner.class)
@SpringBootTest(webEnvironment = SpringBootTest.WebEnvironment.RANDOM_PORT)
@FixMethodOrder(MethodSorters.NAME_ASCENDING)
public class EmployeeControllerComponentTest {
    @Autowired
    WebTestClient webTestClient;
    @MockBean
    private EmployeeRepository employeeRepository;

    @MockBean
    private EmployeeServiceImpl employeeService;


    @Test
    public void saveEmployeeTest () {

        Employee employee = new Employee("5ebbfa934fffbf123cbdf233", "Praneeth", 10000L, null);

        when(employeeService.save(any())).thenReturn(Mono.just(employee));
        webTestClient.post().uri("/employee")
                .body(Mono.just(employee), Employee.class)
                .exchange()
                .expectStatus().isOk()
                .expectBody(Employee.class)
                .consumeWith(resp -> {
                    if (resp.getResponseBody() != null) {
                        assertEquals("Praneeth", resp.getResponseBody().getName());
                    }
                });
    }


    @Test
    public void saveEmployeeTestWithError () {
        Employee employee = new Employee();
        employee.setName("Prasanna");
        employee.setSalary(10000L);

        when(employeeService.save(any())).thenThrow(new RuntimeException());
        this.webTestClient.post().uri("/employee")
                .body(Mono.just(employee), Employee.class)
                .exchange()
                .expectStatus().is5xxServerError();
    }

    @Test
    public void testGetEmployeeDetails () {

        com.example.demo.model.Employee employee = new Employee("5ebbac8c470a341gh872cf3", "Adi", 15000L, null);

        when(employeeService.findAll()).thenReturn(Mono.just(Collections.singletonList(employee)));
        webTestClient.get().uri("/employees")
                .exchange()
                .expectStatus().isOk()
                .expectBodyList(Employee.class)
                .consumeWith(resp -> {
                    if (resp.getResponseBody() != null) {
                        assertEquals("Adi", resp.getResponseBody().get(0).getName());
                    }
                });
    }
    @Test
    public void testGetEmployeeDetailsEmpty () {

        com.example.demo.model.Employee employee = new Employee("5ebbac8c470a341gh872cf3", "Adi", 15000L, null);

        when(employeeService.findAll()).thenReturn(Mono.empty());
        webTestClient.get().uri("/employees")
                .exchange()
                .expectStatus().isOk()
                .expectBodyList(Employee.class)
                .consumeWith(resp -> {
                    if (resp.getResponseBody() != null) {
                        assertEquals("E111", resp.getResponseBody().get(0).getError().getCode());
                    }
                });
    }

    @Test
    public void testGetEmployeeById(){
        when(employeeService.findEmployeeById(anyString())).thenReturn(Mono.empty());

        webTestClient.get().uri("/employee/5ec2580a470a3447d0f90e8c")
                .exchange()
                .expectStatus().isOk()
                .expectBody(Employee.class)
                .consumeWith(resp -> {
                    if (resp.getResponseBody() != null) {
                        assertEquals("E100", resp.getResponseBody().getError().getCode());
                    }
                });
    }

 /*   public void testGetProfileByEmployeeName () {

        Employee employee = new Employee("5ebbfa934fffbf123cbdf233", "Praneeth", 10000L, null);

        when(employeeRepository.save(any())).thenReturn(Mono.just(employee));

        webTestClient.post().uri("/employee/getEmployeeProfileByName/{name}")
                .body(Mono.just(employee), Employee.class)
                .exchange()
                .expectStatus().isOk()
                .expectBody(Employee.class)
                .consumeWith(resp -> {
                    if (resp.getResponseBody() != null) {
                        assertEquals("Praneeth", resp.getResponseBody().getName());
                    }
                });
    }*/
}
