package com.example.demo;

import com.example.demo.Repository.EmployeeRepository;
import com.example.demo.Repository.EmployeeServiceImpl;
import com.example.demo.model.Employee;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.test.context.junit4.SpringRunner;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;
import reactor.test.StepVerifier;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNull;
import static org.mockito.ArgumentMatchers.any;
import static org.mockito.ArgumentMatchers.anyString;
import static org.mockito.Mockito.when;

@SpringBootTest
@RunWith(SpringRunner.class)
public class EmployeeServiceUnitTest {
    @Mock
    private EmployeeRepository empRepo;
    @InjectMocks
    private EmployeeServiceImpl employeeService;


    @Test
    public void testFindEmployeeById(){
        when(empRepo.findById(anyString())).thenReturn(Mono.just(new Employee("5ebbfa934fffbf123cbdf233","Pavani",10100L,null)));


        StepVerifier.create(employeeService.findEmployeeById("5ebbfa934fffbf123cbdf233"))
                .consumeNextWith(employee -> {
                    assertEquals("Pavani",employee.getName());
                })
                .verifyComplete();
    }

    @Test
    public void testFindEmployeeByIdEmpty(){
        when(empRepo.findById(anyString())).thenReturn(Mono.empty());
        StepVerifier.create(employeeService.findEmployeeById("5ebbfa934fffbf123cbdf233"))
                .verifyComplete();
    }

    @Test
    public void testGetEmployees(){
        Flux<Employee> empFlux=getFluxEmployee();
        when(empRepo.findAll()).thenReturn(empFlux);
        StepVerifier.create(employeeService.findAll())
                .expectSubscription()
                .expectNextMatches(employees -> {
                    assertEquals("1",employees.get(0).getId());
                    return true;
                })
                .verifyComplete();
    }

    @Test
    public void testGetEmployeesWithEmpty(){

        when(empRepo.findAll()).thenReturn(Flux.empty());
        StepVerifier.create(employeeService.findAll())
                .expectNextMatches(e-> {

                    return true;
                })
                .verifyComplete();
    }

    private Flux<Employee> getFluxEmployee () {
        return Flux.just(new Employee("1","abc",10000L,null));
    }

    @Test
    public void testSaveEmployee(){
        when(empRepo.save(any())).thenReturn(Mono.just(new Employee("54343ji654b23","Prasanna",10000L,null)));
        StepVerifier.create(employeeService.save(getEmployee()))
                .expectNextMatches(employee -> {
                    assertEquals("Prasanna",employee.getName());
                    return true;
                }).verifyComplete();
    }

    private Employee getEmployee () {
        Employee employee = new Employee();
        employee.setSalary(10000L);
        employee.setName("Pranna");
        //employee.setId("5hfdjhf58899snz");
        return employee;
    }
}
